#coding=utf-8
'''
修改网上的github代码
相关连接为https://pypi.python.org/pypi/ClassicUPS
没事可以提交同步一下~么么哒
@author: totti
'''
import json

import pytz
from datetime import datetime

CEB_TYPE  = {
    'CEB201' : '商品备案（进口）',
    'CEB301' : '电子订单（进口）',
    'EPTRET' : '平台回执',
    'CIQRET' : '检验检疫回执',
    'LOCKED' : '检验检疫锁定指令',
    'UNLOCK	' : '检验检疫解锁指令',
}

RECEIVER_SEND = {
    'EPORT' : '南方平台',
    'SZCIQ' : '深圳检验检疫',
}
RECEIVER_DEPARTMENT = {
    'C' : '海关',
    'Q' : '检验检疫',
    'M' : '市场监管',
    'F' : '外汇',
    'T' : '国税'
}


class CEB(object):
    """
    报关的时候需要的信息
    报文唯一编号
    报文类型
    企业机构代码
    """
    message_type = ''

    org_code = ''

    sender_id = ''
    receiver_id = ''
    receiver_department = ''


    send_time = ''

    version = '1.0'
    signer_info = ''
    sign = ''


        # <MessageHead>
        # </MessageHead>
        # <MessageBody>
        # </MessageBody>

    def __init__(self, message_type, org_code, sender_id, receiver_id,
                 receiver_department, version='1.0', sign = '', signer_info = ''):


        self.message_type = message_type
        self.org_code = org_code
        self.sender_id = sender_id
        self.receiver_id = receiver_id
        self.receiver_department = receiver_department
        self.send_time = datetime.now(pytz.UTC).strftime('%Y%m%d%H%M%S%f')[:-3]
        self.version = version
        self.signer_info = signer_info
        self.sign = sign

    def _generate_xml(self, sign_bool = True , **param):
        xml = '''
        <?xml version="1.0" encoding="utf-8"?>
        <%(message_type)s xmlns = 'ceb'>
        %(MessageHead)s
        %(MessageBody)s
        </%(message_type)s>
        '''
        return xml % { 'message_type': self.get_message_type(), 'MessageHead': self.get_message_head(sign_bool), 'MessageBody': self.get_message_body(**param),}

    def get_message_type(self, type = '出口'):
        return 'CEB201Message'
        # return CEB_TYPE[type] + 'Message'

    def get_message_head(self, sign_bool = True):
        """
        :param sign_bool:是否需要singer信息
        :return:返回meesage_head的xml格式的字符串
        """

        xml = """
            <MessageHead>
                <MessageID>%(MessageID)s</MessageID>
                <MessageType>%(MessageType)s</MessageType>
                <OrgCode>%(OrgCode)s</OrgCode>
                <SenderID>%(SenderID)s</SenderID>
                <ReceiverID>%(ReceiverID)s</ReceiverID>
                <ReceiverDepartment>%(ReceiverDepartment)s</ReceiverDepartment>
                <SendTime>%(SendTime)s</SendTime>
                <Version>%(Version)s</Version>
        """
        return_xml =  xml % {
            'MessageID' : self.get_message_id(),
            'MessageType' : self.message_type,
            'OrgCode' : self.org_code,
            'SenderID' : self.sender_id,
            'ReceiverID' : self.receiver_id,
            'ReceiverDepartment' : self.receiver_department,
            'SendTime' : self.send_time,
            'Version' : self.version,
        }
        if sign_bool:
            return_xml = return_xml + """
                <SignerInfo>%(SignerInfo)s</SignerInfo>
                <Sign>%(Sign)s</Sign>
            </MessageHead>
            """% {
                'SignerInfo' : self.signer_info,
                'Sign' : self.sign,
            }
        else:
            return_xml = return_xml + """
            </MessageHead>
            """

        return return_xml

    def get_message_body(self):
        """
        :return:由各子类来拼凑
        """
        pass

    def get_message_id(self):
        return self.message_type + str(9).zfill(9) + self.send_time + str(4).zfill(4)

    def get_xml(self , ** param):
        xml = self._generate_xml(**param)
        return xml

class CEB_Product(CEB):
    """
    产品相关的报关信息
    """

    def get_message_body(self, **head_params):
        xml_head = """
            <MessageBody>
                <GoodsHead>
                    <appType>%(appType)s</appType>
                    <appTime>%(appTime)s</appTime>
                    <appStatus>%(appStatus)s</appStatus>
                    <appUid>%(appUid)s</appUid>
                    <appUname>%(Sign)s</appUname>
                    <ebpCode>%(ebpCode)s</ebpCode>
                    <ebpName>%(ebpName)s</ebpName>
                    <ebcCode>%(ebcCode)s</ebcCode>
                    <ebcName>%(ebcName)s</ebcName>
                    <agentCode>%(agentCode)s</agentCode>
                    <agentName>%(agentName)s</agentName>
                </GoodsHead>
                """ % head_params
        xml_goods = """
                <GoodsList>
                """
        for good in head_params['goods']:
            xml_goods +="""
                    <gnum>%(gnum)s</gnum>
                    <itemNo>%(itemNo)s</itemNo>
                    <gno>%(gno)s</gno>
                    <ciqGno>%(ciqGno)s</ciqGno>
                    <codeTs>%(codeTs)s</codeTs>
                    <gcode>%(gcode)s</gcode>
                    <gname>%(gname)s</gname>
                    <gmodel>%(gmodel)s</gmodel>
                    <ciqGmodel>%(ciqGmodel)s</ciqGmodel>
                    <element>%(element)s</element>
                    <barCode>%(barCode)s</barCode>
                    <brand>%(brand)s</brand>
                    <unit>%(unit)s</unit>
                    <currency>%(currency)s</currency>
                    <price>%(price)s</price>
                    <originCountry >%(originCountry )s</originCountry >
                    <inspectionFlag>%(inspectionFlag)s</inspectionFlag>
                    <giftFlag>%(giftFlag)s</giftFlag>
                    <supplierName>%(supplierName)s</supplierName>
                    <supplierCountry>%(supplierCountry)s</supplierCountry>
                    <enterprisesName>%(enterprisesName)s</enterprisesName>
                    <enterprisesCountry>%(enterprisesCountry)s</enterprisesCountry>
                    <standards>%(standards)s</standards>
                    <certification>%(certification)s</certification>
                    <supervisionFlag>%(supervisionFlag)s</supervisionFlag>
                    <foodEnterpriseNumber>%(foodEnterpriseNumber)s</foodEnterpriseNumber>
                    <warningFlag>%(warningFlag)s</warningFlag>
                    <ImageList>
                        <imageType>%(imageType)s</imageType>
                        <imageName>%(imageName)s</imageName>
                        <image>%(image)s</image>
                    </ImageList>
                    <AttachmentList>
                        <attachmentType>%(attachmentType)s</attachmentType>
                        <attachmentName>%(attachmentName)s</attachmentName>
                        <attachment>%(attachment)s</attachment>
                    </AttachmentList>
            """ % good

        xml_end = """
                </GoodsList>
            </MessageBody>
            """
        return xml_head + xml_goods + xml_end



def CEB_Order(CEB):
    """
    :param CEB:
    :return:订单报关的拼凑
    """


    def get_message_body(self, **params):
        xml = """
            <MessageBody>
                <OrderBillHead>
                    <appType>%(appType)s</appType>
                    <appTime>%(appTime)s</appTime>
                    <appStatus>%(appStatus)s</appStatus>
                    <appUid>%(appUid)s</appUid>
                    <appUname>%(appUname)s</appUname>
                    <orderNo>%(orderNo)s</orderNo>
                    <ebpCode>%(ebpCode)s</ebpCode>
                    <ebpName>%(ebpName)s</ebpName>
                    <ebcCode>%(ebcCode)s</ebcCode>
                    <ebcName>%(ebcName)s</ebcName>
                    <agentCode>%(agentCode)s</agentCode>
                    <agentName>%(agentName)s</agentName>
                    <goodsValue>%(goodsValue)s</goodsValue>
                    <freight>%(freight)s</freight>
                    <currency>%(currency)s</currency>
                    <cosignee>%(cosignee)s</cosignee>
                    <cosigneeAddress>%(cosigneeAddress)s</cosigneeAddress>
                    <cosigneeTelephone>%(cosigneeTelephone)s</cosigneeTelephone>
                    <cosigneeCountry>%(cosigneeCountry)s</cosigneeCountry>
                    <ProAmount>%(ProAmount)s</ProAmount>
                    <ProRemark>%(ProRemark)s</ProRemark>
                    <note>%(note)s</note>
                </OrderBillHead>
                <OrderBillList>
                    <gnum>%(gnum)s</gnum>
                    <itemNo>%(itemNo)s</itemNo>
                    <gno>%(gno)s</gno>
                    <ciqGno>%(ciqGno)s</ciqGno>
                    <gcode>%(gcode)s</gcode>
                    <gname>%(gname)s</gname>
                    <gmodel>%(gmodel)s</gmodel>
                    <ciqGmodel>%(ciqGmodel)s</ciqGmodel>
                    <barCode>%(barCode)s</barCode>
                    <brand>%(brand)s</brand>
                    <unit>%(unit)s</unit>
                    <currency>%(currency)s</currency>

                    <qty>%(qty)s</qty>
                    <price>%(price)s</price>
                    <total>%(total)s</total>
                    <giftFlag>%(giftFlag)s</giftFlag>
                    <note>%(note)s</note>
                </OrderBillList>
            </MessageBody>
        """
        return xml % params


def CIQ_Retrun(CEB):
    """
    :param CEB:
    :return:订单报关的拼凑
    """


    def get_message_body(self, **params):
        xml = """
            <MessageBody>
                <ResultMSG>
                    <ciqCode>%(ciqCode)s</ciqCode>
                    <supervisionCode>%(supervisionCode)s</supervisionCode>
                    <billNo>%(billNo)s</billNo>
                    <ciqNo>%(ciqNo)s</ciqNo>
                    <resultType>%(resultType)s</resultType>
                    <resultDetail>%(resultDetail)s</resultDetail>
                </ResultMSG>
            </MessageBody>
        """
        return xml % params


def CIQ_Locked(CEB):
    """
    :param CEB:
    :return:订单报关的拼凑
    """


    def get_message_body(self, **params):
        xml = """
            <MessageBody>
                <OrderMSG>
                    <ciqCode>%(ciqCode)s</ciqCode>
                    <supervisionCode>%(supervisionCode)s</supervisionCode>
                    <orderNo>%(orderNo)s</orderNo>
                    <contents>%(contents)s</contents>
                    <ciqNo>%(ciqNo)s</ciqNo>
                    <detail>%(detail)s</detail>
                </OrderMSG>
            </MessageBody>
        """
        return xml % params

def EPT_Return(CEB):
    """
    :param CEB:
    :return:订单报关的拼凑
    """

    def get_message_body(self, **params):
        xml = """
            <MessageBody>
                <ResultMSG>
                    <billNo>%(billNo)s</billNo>
                    <resultType>%(resultType)s</resultType>
                    <resultDetail>%(resultDetail)s</resultDetail>
                </ResultMSG>
            </MessageBody>
        """
        return xml % params

if __name__=='__main__':

    ebay_order = Product('message_type','org_code', 'sender_id', 'receiver_id', 'receiver_department')
    print ebay_order.get_xml()
